#!/bin/python3
import os
import sys
from bs4 import BeautifulSoup
import requests
import json
import time
import sys
from datetime import datetime
import re
#url = sys.argv[1]

green = '\033[92m'
blue = '\033[94m'
yellow = '\033[93m'
reset = '\033[0m'
red = '\033[31m'



url = 'https://androidfilehost.com/?w=files&flid=35936'
def get_response(url):
    while True:
        html = requests.get(url)
        if '<b>Warning</b>' not in html.content.decode('utf-8') and 'Bad gateway' not in html.content.decode('utf-8'):
            break
        else:
            print('retrying')
    print('request to ', url, ' sucessful')
    soup = BeautifulSoup(html.content, "html.parser")
    return soup, html

def parse_downloads(downloads_str):
    downloads_multipliers = {'K': 1000, 'M': 1000000, 'B': 1000000000}

    # Remove specific characters
    for char in [' ', '+', ',', '-', '<', '=', '>']:
        downloads_str = downloads_str.replace(char, '')

    if downloads_str[-1] in downloads_multipliers:
        return int(float(downloads_str[:-1]) * downloads_multipliers[downloads_str[-1]])
    else:
        return int(downloads_str)

def parse_size(size_str):
    size_multipliers = {
        'B': 1,
        'KB': 2**10,
        'MB': 2**20,
        'GB': 2**30,
        'TB': 2**40,
        'PB': 2**50,
        'EB': 2**60,
        'ZB': 2**70,
        'YB': 2**80,
        'RB': 2**90,
        'QB': 2**100
    }
    pattern = r'^\d+(\.\d+)?B$'

    if re.match(pattern, size_str):
        size_value_str = re.sub("[^0-9]", "", size_str)
        return float(size_value_str)
    else:

        # Split the string into value and unit
        size_value_str, size_unit_str = size_str[:-2], size_str[-2:]
#        print('size_value_str = ', size_value_str)
        if size_unit_str in size_multipliers:
            return float(size_value_str) * size_multipliers[size_unit_str]
        else:
            return float(size_value_str)

def is_md5(s):
    return re.fullmatch(r'[A-Fa-f0-9]{32}', s) is not None

def list_files(soup, current_dir):
    file_name_ul = soup.find_all('ul', class_='list-group files')
    links = []
    for ul in file_name_ul:
        a_tags = ul.find_all('a')
        for a_tag in a_tags:
#           print(a_tag)
            if a_tag and a_tag.get('href').startswith('/?fid=') and not a_tag.find('small'):
                links.append(a_tag)
#                print(a_tag)
#        print(links)
        if links:
            for link in links:
                link = 'https://androidfilehost.com' + link.get('href')
                print(f"{green}{link}{reset}")
                url = link
                soup, html = get_response(url)
#                print(soup)

                file_id = soup.find("input", {"name": "fid"})["value"]
                file_info_div = soup.find("div", class_="col-xs-8 col-xs-offset-4 file-info")
                filename = file_info_div.h3.text.strip().replace('/','-')
                if len(filename) >= 255:
                    filename = filename[:250]

                file_name, file_extension = os.path.splitext(filename)
                category_link = file_info_div.find("a", href=lambda href: href and "/?w=developers&did=" in href)
                category_id = category_link["href"].split("=")[-1] if category_link else None
                category = category_link.text.strip() if category_link else None
                user_link = file_info_div.find("a", href=lambda href: href and "/?w=profile&uid=" in href)
                user_id = user_link["href"].split("=")[-1] if user_link else None
                user = user_link.text.strip() if user_link else None
                downloads = soup.find("img", src="/images/d-s-downloads.png").find_next("span").text.strip()[:-9]
                downloads_approx = parse_downloads(downloads)
                size = soup.find("img", src="/images/d-s-size.png").find_next("span").text.strip()[:-4]
                print('size = ' + size)
                if size == '0B':
                    size_approx = 0
                else:
                    size_approx = parse_size(size)
                md5 = soup.find("img", src="/images/d-s-md5.png").find_next("span").text.strip()[:-3]
                md5_check = is_md5(md5)
                md5 = md5 if md5_check else 'Unknown'
                upload_date = soup.find("img", src="/images/d-s-uploadeddate.png").find_next("span").text.strip()[:-11]
                upload_date_object = datetime.strptime(upload_date, '%b %d, %Y | %I:%M%p')
                year = upload_date_object.year
                year = f"{year:04d}"
                month = upload_date_object.month
                month = f"{month:02d}"
                day = upload_date_object.day
                day = f"{day:02d}"
                hour = upload_date_object.hour
                hour = f"{hour:02d}"
                minute = upload_date_object.minute
                minute = f"{minute:02d}"
                upload_date_iso_8601 = f"{year}-{month}-{day}T{hour}:{minute}±??:??"
                weekdays = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']
                weekday_name = weekdays[upload_date_object.weekday()]
                months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
                month_name = months[int(month) - 1]
                upload_date_rfc_5322 = f"{weekday_name}, {day} {month_name} {year} {hour}:{minute}:?? ±????"
                upload_date_rfc_3339 = f"{year}-{month}-{day} {hour}:{minute}:??±??:??"
                description = soup.find("h4", string="Description:").find_next("span").text.strip()
                additional_information = soup.find("h4", string="Additional Information:").find_next("span").text.strip() 

                file_info = {
                    "File ID": file_id,
                    "File name": file_name,
                    "File extension": file_extension,
                    "Category ID": category_id,
                    "Category": category,
                    "User ID": user_id,
                    "User": user,
                    "Downloads": downloads,
                    "Downloads (approximate)": downloads_approx,
                    "Size": size,
                    "Size (approximate)": size_approx,
                    "MD5": md5,
                    "Upload Date": upload_date,
                    "Upload Date (ISO 8601)": upload_date_iso_8601,
                    "Upload Date (RFC 5322)": upload_date_rfc_5322,
                    "Upload Date (RFC 3339)": upload_date_rfc_3339,
                    "Description": description,
                    "Additional Information": additional_information
                }
                json_name = working_dir + '/'+ file_id + '_' + file_name + '.json'
                print(json_name)
                print(json.dumps(file_info, indent=4))
                with open(json_name, 'w+') as f:
                    f.write(json.dumps(file_info, indent=4))


    if not links:
        print(f"{yellow}{'no files found'}{reset}")

def list_dirs(soup):
    dir_lists = soup.find_all('ul', class_='list-group folders')
    dir_links =[]
    for dir_link in dir_lists:
        a_elements = dir_link.find_all('a')
        for a_element in a_elements:
            dir_links.append(a_element)
        return dir_links

def get_current_dir(soup):
    f_title_div = soup.find('div', class_='f-title')
    #print(f_title_div)
    if f_title_div is not None:
        texts = f_title_div.stripped_strings
        formatted_text = ' / '.join(text.replace(' ', '_') for text in texts)
        current_dir = formatted_text.replace(' ', '')
        return current_dir
    else:
        print(f"{red}{'div with class f-title not found in '}{url}{reset}")
        print(soup)
        sys.exit()

def scrape(link, username):
    url = 'https://androidfilehost.com' + link
    soup, html = get_response(url)
    if 'Access Denied' in html.content.decode('utf-8'):
        print (f"{red}{'access denied'}{reset}")
        return
    if 'file in question has been deleted from the site and servers' in html.content.decode('utf-8'):
        print (f"{red}{'not found'}{reset}")
        return
    current_dir = get_current_dir(soup)
    working_dir = '/srv/big_vault/nicholas/scrapers/AFH/out_fixed/' + device + '/' + username + '/' + current_dir
    if not os.path.exists(working_dir):
        os.makedirs(working_dir)
    os.chdir(working_dir)
    print(f"{green}{working_dir}{reset}")
    list_files(soup, working_dir)
    dir_links = list_dirs(soup)
    if dir_links is not None:
        for link in dir_links:
            link = link.get('href')
            print(f"{yellow}{'scraping '}{link}{reset}")
            scrape(link, username)
    else:
        print(f"{yellow}{'continuing loop url is '}{url}{reset}")


#soup, html = get_response(url)
devices = os.listdir('/srv/big_vault/nicholas/scrapers/AFH/out_fixed')
#print(devices)
for device in devices:
    print('device: ', device)
    if device == 'get_metadata.py':
        continue
    os.chdir(device)
    print(device)
    if 'done_scraping_metadata' in os.listdir('/srv/big_vault/nicholas/scrapers/AFH/out_fixed/' + device):
        print(f"{green}{device} already scraped{reset}")
        os.chdir('/srv/big_vault/nicholas/scrapers/AFH/out_fixed')
        continue
    #print(os.listdir())



    with open('dev_list.json') as f:
        data = json.load(f)
        for dev in data['devs']:
            username = dev['username']
            url = dev['url']
            print("Username:", username)
            print('url', url)
            print()
            if os.path.isfile('/srv/big_vault/nicholas/scrapers/AFH/out_fixed/' + device + '/' + username + '/' + 'done_scraping_dev'):
                print(f"{green}{dev} already scraped{reset}")
                continue
            soup, html = get_response(url)
            if 'file in question has been deleted from the site and servers' in html.content.decode('utf-8'):
                print (f"{red}{'not found'}{reset}")
                continue
            current_dir = get_current_dir(soup)
            working_dir = '/srv/big_vault/nicholas/scrapers/AFH/out_fixed/' + device + '/' + username + '/' + current_dir
            if not os.path.exists(working_dir):
                os.makedirs(working_dir + '/root')
            list_files(soup, working_dir)
            dir_links = list_dirs(soup)
            #print(f"{green}{dir_links}{reset}")
            print(working_dir)
            print(f"{blue}{'--------------------------------------------------------'}{reset}")
            if dir_links is not None:
                for link in dir_links:
                    link = link.get('href')
                    scrape(link, username)
            done_scraping_dev = open('/srv/big_vault/nicholas/scrapers/AFH/out_fixed/' + device + '/' + username + '/' + 'done_scraping_dev', 'a')
            print(f"{green}done scraping {username}{reset}")
    print(f"{green}done scraping device{reset}")
    done_scraping_metadata = open('/srv/big_vault/nicholas/scrapers/AFH/out_fixed/' + device + '/' + 'done_scraping_metadata', 'a')

    os.chdir('/srv/big_vault/nicholas/scrapers/AFH/out_fixed')
